/*
    `yy' - A lightshow-like program that graphically represents sound.
    Copyright (C) 1998  Nikos Gorogiannis <kgor@softlab.ece.ntua.gr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
    See file `copyright' for details.
*/

#include <stdlib.h>
#include <vga.h>

#include "framebuffer.hh"
#include "color_mix.hh"
#include "def_shared.hh"


void FrameBuffer::create(void)
{
	int gMode;
	
	switch(requestedMode)
	{
		case 1:
			gMode = G320x240x256;
			break;

		case 2:
			gMode = G320x400x256;
			break;

		case 3:
			gMode = G360x480x256;
			break;

		case 4:
			gMode = G640x480x256;
			break;

		case 5:
			gMode = G800x600x256;
			break;

		case 6:
			gMode = G1024x768x256;
			break;

		case 7:
			gMode = G1280x1024x256;
			break;

		default:
		case 0:
			gMode = G320x200x256;
			break;
	}
		
	chk( vga_init() );
	chk( vga_setmode(gMode) );

	if(gl_setcontextvga(gMode)!=0)
		error("gl_setcontextvga");

	screen = gl_allocatecontext();
	chkVal(screen, NULL);
	
	gl_getcontext(screen);

	if(gl_setcontextvgavirtual(gMode)!=0)
		error("gl_setcontextvgavirtual");

	screenSize = WIDTH*HEIGHT*BYTESPERPIXEL;
	width = WIDTH;
	height = HEIGHT;
}

void FrameBuffer::destroy(void)
{
	free(screen);
	vga_setmode(TEXT);
}

void FrameBuffer::setcolor(int c, int r, int g, int b) const
{
	gl_setpalettecolor(c, r, g, b);
}

void FrameBuffer::copyToScreen(void) const
{
	gl_copyscreen(screen);
}

void FrameBuffer::age(const double aging) const
{
	register Pixel *ptr = (Pixel *) VBUF;
	const int intAging = (int)(aging*65536.0);
	register int bri;
	
	for(register int i=0; i<screenSize; i++, ptr++)
	{
		if(*ptr==0)
			continue;

		bri = ((int)brightness(*ptr) * intAging) >> 16;
		
		if(bri==0)
			*ptr = 0;
		else
			*ptr = pcolor(palette(*ptr), (Pixel) bri);
	}
}

void FrameBuffer::icon
	(const int x, const int y, const int index, const Pixel p, 
	const Pixel c) const
{
	const int size = 2*index+1;

	if(x-index<0)
		return;
	else if(x+index+1 >= width)
		return;

	if(y-index<0)
		return;
	else if(y+index+1 >= height)
		return;

	register Pixel * ptr = (Pixel *)VBUF + (y-index)*width + x-index;

	const int lineAdvance = width - size;

	register int * multiplier = icons.bitmap[index];
	
	for(register int y=0; y<size; y++)
	{
		for(register int x=0; x<size; x++)
		{
			const int value = 
			( (int)c * (*multiplier) ) >> LOG2_MAX_DIVISOR;
			*ptr = colorMixer(pcolor(p, (Pixel) value), *ptr);
			ptr++;
			multiplier++;
		}
			
		ptr += lineAdvance;
	}
}
