%{
#include <stdio.h>
#include <iostream.h>
#include <unistd.h>
#include <sys/time.h>
#include "def_shared.hh"
#include "color_mix.hh"
#include "trans.hh"
#include "icons.hh"

extern unsigned lineCount;

void yyerror(const char *msg)
{
	cerr << '\n' << msg << " at line " << lineCount << '\n';
}

void accurateSleep(const unsigned usecs)
{
	struct timeval tv;
       
	tv.tv_sec = usecs / 1000000;
	tv.tv_usec = usecs % 1000000;
	chk( select(0, NULL, NULL, NULL, &tv) );
}

extern int yylex(void);
extern FILE * yyin;

double secsPerBar = 8.0;
unsigned bpm = 120;
unsigned beatsPerBar = 16;
%}

%union {
	unsigned UnsignedVal;
	double DoubleVal;
}

%token TK_BPM
%token TK_BPB
%token TK_WAITKEY
%token TK_IN
%token TK_ATTENUATION
%token TK_THRESHOLD
%token TK_AGING
%token TK_SLEEP
%token TK_TRANSFORMATION
%token TK_COLOR
%token TK_APERTURE
%token TK_COLOR_MIXER
%token TK_ICONMINSIZE
%token TK_ICONMAXSIZE
%token <UnsignedVal> TK_UNSIGNED
%token <DoubleVal> TK_DOUBLE

%%
inputFile: assignments
;

assignments: /* empty */
	| assignments assignment
;

assignment: TK_BPM '=' TK_UNSIGNED
	{
		bpm = $3;
		secsPerBar = 60.0*(double)beatsPerBar/(double)bpm;
	}
	| TK_BPB '=' TK_UNSIGNED
	{
		beatsPerBar = $3;
		secsPerBar = 60.0*(double)beatsPerBar/(double)bpm;
	}
	| TK_WAITKEY
	{
		char tmp;
		read(0, &tmp, sizeof(tmp));
	}
	| TK_ATTENUATION '=' TK_UNSIGNED
	{
		sharedVars->rAttenuation = (double)$3/100.0;
	}
	| TK_THRESHOLD '=' TK_UNSIGNED
	{
		sharedVars->threshold = (double)$3/100.0;
	}
	| TK_SLEEP TK_DOUBLE
	{
		accurateSleep( (unsigned) (secsPerBar*$2*1000000) );
	}
	| TK_AGING '=' TK_UNSIGNED
	{
		sharedVars->rAging = 1.0 - ((double)$3/100.0);
	}
	| TK_TRANSFORMATION TK_UNSIGNED TK_IN TK_DOUBLE
	{
		const double duration = 1000000*$4*secsPerBar;
		   
		if($2<TRANSFORMATIONS)
		{
			sharedVars->state = STATE_TRANSIT;
			sharedVars->transfIndex2 = $2;
			sharedVars->stateFraction = 0.0;

			while(sharedVars->stateFraction < 1.0)
			{
				accurateSleep((unsigned)(duration/20.0));
				sharedVars->stateFraction += 1.0/20.0;
			}

			sharedVars->state = STATE_STABLE;
			sharedVars->transfIndex1 = $2;
		}
	}
	| TK_COLOR TK_UNSIGNED '<' TK_DOUBLE TK_DOUBLE TK_DOUBLE '>'
	{
		if($2>MAX_PALETTE)
			yyerror("only 4 colors may be specified (0-3)");
		     
		sharedVars->palette[$2].hue = $4;
		sharedVars->palette[$2].saturation = $5;
		sharedVars->palette[$2].value = $6;
		
		sharedVars->paletteChange = 1;
	}
	| TK_APERTURE '=' TK_DOUBLE
	{
		if($3<=0.0)
			yyerror("aperture can only be positive");
		   
		sharedVars->aperture = $3;
	}
	| TK_COLOR_MIXER TK_UNSIGNED
	{
		if($2>=COLOR_MIXERS)
			yyerror("There aren't so many color mixers");
		   
		sharedVars->colorMixerIndex = $2;
	}
	| TK_ICONMINSIZE '=' TK_UNSIGNED
	{
		if($3>=ICONS)
			yyerror("There aren't so many color icons");
		   
		sharedVars->iconMinSize = $3;
	}
	| TK_ICONMAXSIZE '=' TK_UNSIGNED
	{
		if($3>=ICONS)
			yyerror("There aren't so many color icons");
		   
		sharedVars->iconMaxSize = $3;
	}
;

%%
/*
int main(int argc, char **argv)
{
yyin = fopen(argv[1], "r");
return yyparse();
}
*/
