/*
    `yy' - A lightshow-like program that graphically represents sound.
    Copyright (C) 1998  Nikos Gorogiannis <kgor@softlab.ece.ntua.gr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
    See file `copyright' for details.
*/

#include <unistd.h>
#include <linux/soundcard.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <math.h>
#include <string.h>

#include "def_settings.hh"
#include "sema.hh"

#define MAX_TIME_LAG ((double) 0.05)

extern Semaphore sema;

#if SAMPLE_TYPE == S16
#define SAMPLE_FORMAT AFMT_S16_LE
#elif SAMPLE_TYPE == U16
#define SAMPLE_FORMAT AFMT_U16_LE
#elif SAMPLE_TYPE == S8
#define SAMPLE_FORMAT AFMT_S8
#elif SAMPLE_TYPE == U8
#define SAMPLE_FORMAT AFMT_U8
#endif


void recorder(void)
{
	// open sound-device and set parameters
	const int dsp = open("/dev/dsp", O_RDONLY);
	chkVal(dsp, -1);

	int format = SAMPLE_FORMAT;
	chk( ioctl(dsp, SNDCTL_DSP_SETFMT, &format) );
	if(format != SAMPLE_FORMAT)
		error("Audio format not supported by soundcard, reported format="<<format);

	int stereo = 1;
	chk( ioctl(dsp, SNDCTL_DSP_STEREO, &stereo) );
	if(stereo != 1)
		error("Stereo recording not supported by soundcard!");

	int frequency = FREQUENCY;
	chk( ioctl(dsp, SNDCTL_DSP_SPEED, &frequency) );
	if(frequency != FREQUENCY)
		error("Requested frequency not supported by soundcard.");

	// calculation of block-size/(interrupt||dma ?) that sound-driver reads.
	// the number is of the form 0xBBBBLLLL where BBBB is the number of
	// buffers for the sound-driver to allocate. We really don't care for
	// 'old' fragments so we use the minimum value which is 2. 
	
	int requestedFragment = 
		(int) floor( log(MAX_TIME_LAG * FREQUENCY) /
		log(2.0)) + 2;
	
	int fragment = 0x00020000 + requestedFragment;

	chk( ioctl(dsp, SNDCTL_DSP_SETFRAGMENT, &fragment) );
	if( (fragment & 0x0000FFFF) != requestedFragment )
		error("Requested fragment size could not be set.");

	Sample * inputBuffer = new Sample [SAMPLES*2];
	chkVal(inputBuffer, NULL);

	while(1)
	{
		// record
		if( read(dsp, (void *) inputBuffer, BUF_SIZE) != BUF_SIZE )
			error("read(dsp,...) return other than bufSize");

		sema.wait();
		memcpy((void *) buffer, (void *) inputBuffer, BUF_SIZE);
		sema.signal();

		if(sharedVars->terminate)
			break;
	}
	
	delete [] inputBuffer;

	chk( close(dsp) );
}
