/*
    `yy' - A lightshow-like program that graphically represents sound.
    Copyright (C) 1998  Nikos Gorogiannis <kgor@softlab.ece.ntua.gr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
    See file `copyright' for details.
*/

#include <math.h>
#include <stdlib.h>

#include "trans.hh"
#include "def_shared.hh"
#include "def_settings.hh"
#include "function.hh"

static const IntFunction iCos(cos, -3.14, 3.14, 1000);
static const IntFunction iSin(sin, -3.14, 3.14, 1000);

static const double a=16.357, b=0.0578;
static unsigned randomXs[100];

static double maxnote;

void initTransformations(void)
{
	for(unsigned i=0; i<100; i++)
	    randomXs[i] = rand() % MAXX;

	maxnote = log((double)FREQUENCY/2.0/a)/b;
	 
}

unsigned Y1(const TransInput * ti)
{
	return MAXY - (unsigned)(ti->freqIndex*MAXY*2.0/SAMPLES);
}

unsigned X1(const TransInput * ti)
{
	return MAXX/2 + (int)(ti->phi*MAXX/2.0);
}

unsigned X2(const TransInput * ti)
{
	return MAXX/2 + (int)(MAXX/2.0*iSin(ti->phi*2.0*pi2)*ti->freqIndex*2.0/SAMPLES);
}

unsigned Y2(const TransInput * ti)
{
	return MAXY/2 + (int)(MAXY/2.0*iCos(ti->phi*2.0*pi2)*ti->freqIndex*2.0/SAMPLES);
}

unsigned X3(const TransInput * ti)
{
	return MAXX/2 + (int)(MAXX/2.0*ti->phi);
}

unsigned Y3(const TransInput * ti)
{
	const double lnote = log((double)ti->freqIndex*(double)FREQUENCY/(double)SAMPLES/a)/b;
	const double note = fmod(lnote, 12.0);
	return 199 - (unsigned)(MAXY*note/12.0);
//	return 199 - (unsigned)(199.0*lnote/maxnote);
}

unsigned X4(const TransInput * ti)
{
	return rand()%MAXX;
}

unsigned X5(const TransInput * ti)
{
	return MAXX - (int)(MAXX * ti->freqIndex*2.0/SAMPLES);
}

unsigned Y5(const TransInput * ti)
{
	return MAXY/2 - (int)(MAXY/2.0*ti->phi);
}

unsigned Y6(const TransInput * ti)
{
	return (unsigned)(MAXY*ti->freqIndex*2.0/SAMPLES);
}

unsigned X6(const TransInput * ti)
{
	return MAXX/2 - (int)(MAXX/2.0*ti->phi);
}

unsigned X7(const TransInput * ti)
{
	return (unsigned)(MAXX * ti->freqIndex*2.0/SAMPLES);
}

unsigned Y7(const TransInput * ti)
{
	return MAXY/2 + (int)(MAXY/2.0*ti->phi);
}

unsigned Y8(const TransInput * ti)
{
	const double r = 1 - fabs(ti->phi);
	const double theta = 2.0*pi2 * 2.0*ti->freqIndex/SAMPLES - pi2;

	return MAXY/2 - (unsigned) (MAXY/2.0*r*iSin(theta));
}

unsigned X8(const TransInput * ti)
{
	const double r = 1 - fabs(ti->phi);
	const double theta = 2.0*pi2 * 2.0*ti->freqIndex/SAMPLES - pi2;
	const int x = (int) (MAXX/2.0*r*iCos(theta));
	if(ti->phi<0.0)
		return x;
	else
		return MAXX - x;
}


const Transformation transformations[TRANSFORMATIONS] =
{	
	{ //0
		X1,
		Y1
	},
	{ //1
		X2,
		Y2
	},
	{ //2
		X1,
		Y2
	},
	{ //3
		X3,
		Y1
	},
	{ //4
		X1,
		Y3
	},
	{ //5
		X4,
		Y1
	},
	{ //6
		X1,
		Y2
	},
	{ //7
		X5,
		Y5
	},
	{ //8
		X6,
		Y6
	},
	{ //9
		X7,
		Y7
	},
	{ //10
	  	X8,
		Y8
	}
};
